@@ -1,4 +1,4 @@
-/* $OpenBSD: canohost.h,v 1.10 2009/02/12 03:00:56 djm Exp $ */
+/* $OpenBSD: canohost.h,v 1.12 2016/03/07 19:02:43 djm Exp $ */
 
 /*
  * Author: Tatu Ylonen <ylo@cs.hut.fi>
@@ -12,18 +12,15 @@
  * called by a name other than "ssh" or "Secure Shell".
  */
 
-const char	*get_canonical_hostname(int);
-const char	*get_remote_ipaddr(void);
-const char	*get_remote_name_or_ip(u_int, int);
+#ifndef _CANOHOST_H
+#define _CANOHOST_H
 
 char		*get_peer_ipaddr(int);
 int		 get_peer_port(int);
 char		*get_local_ipaddr(int);
 char		*get_local_name(int);
+int		get_local_port(int);
 
-int		 get_remote_port(void);
-int		 get_local_port(void);
-int		 get_sock_port(int, int);
-
+#endif /* _CANOHOST_H */
 
 void		 ipv64_normalise_mapped(struct sockaddr_storage *, socklen_t *);
