@@ -40,6 +40,7 @@
 #include <arpa/inet.h>
 
 #include <errno.h>
+#include <stdlib.h>
 #include <string.h>
 
 #define STARTPORT 600
@@ -64,6 +65,7 @@ bindresvport_sa(int sd, struct sockaddr *sa)
 	if (sa == NULL) {
 		memset(&myaddr, 0, sizeof(myaddr));
 		sa = (struct sockaddr *)&myaddr;
+		salen = sizeof(myaddr);
 
 		if (getsockname(sd, sa, &salen) == -1)
 			return -1;	/* errno is correctly set */
@@ -89,7 +91,7 @@ bindresvport_sa(int sd, struct sockaddr *sa)
 
 	port = ntohs(*portp);
 	if (port == 0)
-		port = (arc4random() % NPORTS) + STARTPORT;
+		port = arc4random_uniform(NPORTS) + STARTPORT;
 
 	/* Avoid warning */
 	error = -1;
