@@ -1,11 +1,14 @@
-#	$OpenBSD: conch-ciphers.sh,v 1.4 2019/07/05 04:12:46 dtucker Exp $
+#	$OpenBSD: conch-ciphers.sh,v 1.7 2023/10/26 12:44:07 dtucker Exp $
 #	Placed in the Public Domain.
 
 tid="conch ciphers"
 
 if test "x$REGRESS_INTEROP_CONCH" != "xyes" ; then
-	echo "conch interop tests not enabled"
-	exit 0
+	skip "conch interop tests not enabled"
+fi
+
+if ! [ -t 0 ]; then
+	skip "conch interop tests requires a controlling terminal"
 fi
 
 start_sshd
@@ -16,7 +19,7 @@ for c in aes256-ctr aes256-cbc aes192-ctr aes192-cbc aes128-ctr aes128-cbc \
 	rm -f ${COPY}
 	# XXX the 2nd "cat" seems to be needed because of buggy FD handling
 	# in conch
-	${CONCH} --identity $OBJ/ssh-rsa --port $PORT --user $USER -e none \
+	${CONCH} --identity $OBJ/ssh-ed25519 --port $PORT --user $USER -e none \
 	    --known-hosts $OBJ/known_hosts --notty --noagent --nox11 -n \
 	    127.0.0.1 "cat ${DATA}" 2>/dev/null | cat > ${COPY}
 	if [ $? -ne 0 ]; then
