@@ -10,9 +10,7 @@
 #include <sys/types.h>
 #include <signal.h>
 #include <stdio.h>
-#ifdef HAVE_STDINT_H
 #include <stdint.h>
-#endif
 #include <stdlib.h>
 #include <string.h>
 
