@@ -1,4 +1,4 @@
-/* $OpenBSD: sftp.c,v 1.240 2025/03/28 06:04:07 dtucker Exp $ */
+/* $OpenBSD: sftp.c,v 1.245 2025/10/02 04:23:11 djm Exp $ */
 /*
  * Copyright (c) 2001-2004 Damien Miller <djm@openbsd.org>
  *
@@ -1837,7 +1837,7 @@ complete_display(char **list, u_int len)
 
 /*
  * Given a "list" of words that begin with a common prefix of "word",
- * attempt to find an autocompletion to extends "word" by the next
+ * attempt to find an autocompletion that extends "word" by the next
  * characters common to all entries in "list".
  */
 static char *
@@ -2316,6 +2316,8 @@ interactive_loop(struct sftp_conn *conn, char *file1, char *file2)
 	free(startdir);
 	free(conn);
 
+	if (hl != NULL)
+		history_end(hl);
 	if (el != NULL)
 		el_end(el);
 
@@ -2613,7 +2615,7 @@ main(int argc, char **argv)
 	} else {
 		if ((r = argv_split(sftp_direct, &tmp, &cpp, 1)) != 0)
 			fatal_r(r, "Parse -D arguments");
-		if (cpp[0] == 0)
+		if (cpp[0] == NULL)
 			fatal("No sftp server specified via -D");
 		connect_to_server(cpp[0], cpp, &in, &out);
 		argv_free(cpp, tmp);
